import crypto from 'node:crypto';
import { defineNuxtModule, createResolver, addPlugin, addServerHandler, addImportsDir, addImports, addComponentsDir } from '@nuxt/kit';
import defu from 'defu';

const name = "@devite/nuxt-sanity";
const version = "2.2.1";

const module = defineNuxtModule({
  meta: {
    name,
    version,
    configKey: "sanity"
  },
  defaults: {},
  async setup(options, nuxt) {
    const { resolve } = createResolver(import.meta.url);
    const moduleConfig = defu(nuxt.options.runtimeConfig.public.sanity, {
      projectId: options.projectId,
      dataset: options.dataset || "production",
      minimalClient: options.minimalClient || false,
      useCdn: options.useCdn || true,
      apiVersion: options.apiVersion || "2024-08-08",
      visualEditing: options.visualEditing || null
    });
    nuxt.options.build.transpile.push("@sanity/core-loader");
    if (options.visualEditing) {
      const previewMode = moduleConfig.visualEditing.previewMode !== false;
      const visualEditingConfig = defu(moduleConfig.visualEditing, {
        mode: "live-visual-editing",
        previewMode: previewMode ? defu(moduleConfig.visualEditing.previewMode, {
          enable: "/_sanity/preview/enable",
          disable: "/_sanity/preview/disable"
        }) : false,
        previewModeId: previewMode ? crypto.randomBytes(16).toString("hex") : "",
        proxyEndpoint: "/_sanity/fetch",
        stega: true,
        zIndex: 100,
        token: ""
      });
      if (!moduleConfig.visualEditing.token?.length)
        console.warn('Visual editing requires a token with "read" access');
      if (!visualEditingConfig.studioUrl)
        console.warn("Visual editing requires a studio URL");
      if (visualEditingConfig.mode === "live-visual-editing" && !visualEditingConfig.stega)
        console.warn('Visual Editing requires "stega" to be enabled in "live-visual-editing" mode');
      nuxt.options.build.transpile.push("@sanity/preview-url-secret", "async-cache-dedupe");
      nuxt.options.vite.resolve = defu(nuxt.options.vite.resolve, { dedupe: ["@sanity/client"] });
      nuxt.options.vite.optimizeDeps = defu(nuxt.options.vite.optimizeDeps, {
        include: [
          `${name} > @sanity/visual-editing > @sanity/mutate > lodash/groupBy.js`,
          `${name} > @sanity/visual-editing > react`,
          `${name} > @sanity/visual-editing > react/jsx-runtime`,
          `${name} > @sanity/visual-editing > react-dom`,
          `${name} > @sanity/visual-editing > react-dom/client`,
          `${name} > @sanity/visual-editing > react-compiler-runtime`,
          "@sanity/client"
        ]
      });
      addPlugin({ src: resolve("runtime/plugins/visual-editing") });
      if (previewMode) {
        addServerHandler({
          method: "get",
          route: visualEditingConfig.previewMode.enable,
          handler: resolve("runtime/server/routes/preview/enable")
        });
        addServerHandler({
          method: "get",
          route: visualEditingConfig.previewMode.disable,
          handler: resolve("runtime/server/routes/preview/disable")
        });
      }
      addServerHandler({
        method: "post",
        route: visualEditingConfig.proxyEndpoint,
        handler: resolve("runtime/server/routes/proxy")
      });
      moduleConfig.visualEditing = visualEditingConfig;
    }
    nuxt.options.runtimeConfig.sanity = moduleConfig;
    nuxt.options.runtimeConfig.public.sanity = defu(nuxt.options.runtimeConfig.public.sanity, {
      projectId: moduleConfig.projectId,
      dataset: moduleConfig.dataset,
      minimalClient: moduleConfig.minimalClient,
      useCdn: moduleConfig.useCdn,
      apiVersion: moduleConfig.apiVersion,
      perspective: "raw",
      /* Visual Editing */
      token: options.token || "",
      withCredentials: options.withCredentials || false,
      stega: moduleConfig.visualEditing && moduleConfig.visualEditing.stega !== false && moduleConfig.visualEditing.previewMode !== false && {
        enabled: true,
        studioUrl: moduleConfig.visualEditing.studioUrl
      } || {},
      visualEditing: moduleConfig.visualEditing
    });
    addImportsDir(resolve("runtime/client"));
    addImports(
      [
        // composables
        { name: "useSanityQuery", from: resolve("runtime/composables/useSanityQuery") },
        { name: "useLazySanityQuery", from: resolve("runtime/composables/useLazySanityQuery") },
        // helper methods
        { name: "groq", from: resolve("runtime/utils/groq") },
        { name: "resolveImageAssetById", from: resolve("runtime/utils/resolveImageAssetById") },
        { name: "resolveInternalLink", from: resolve("runtime/utils/resolveInternalLink") },
        // projections
        ...[
          "IMAGE_ASSET_PROJECTION",
          "IMAGE_WITHOUT_PREVIEW_PROJECTION",
          "IMAGE_WITH_PREVIEW_PROJECTION",
          "LINK_PROJECTION"
        ].map((field) => ({
          name: field,
          from: resolve("runtime/utils/projections")
        }))
      ]
    );
    await addComponentsDir({ path: resolve("runtime/components") });
    await addComponentsDir({
      path: "~/sanity",
      global: true,
      prefix: "Sanity",
      pathPrefix: false
    });
  }
});

export { module as default };
